% function FS=DFST(lf)
% INPUT: lf, lightfield with Nxy x Nxy microlenses, Nuv x Nuv
% pixels behind each microlens and nbands per pixel (nbands=3 for color images) 
% stored in a [Nxy,Nxy,Nuv,Nuv,nbands] array . 
% lf can be obtained from an image using the read_lightfield3 function.
% OUTPUT: FS, focal stack  from the lightfield computed with the Discrete
% Focal Stack Transform (DFST) stored in a [Nxy,Nxy,nbands,Nplanes] array with
% Nplanes=Nuv+(Nuv/2==floor(Nuv/2)).


function FS=DFST(lf)

[Nxy,Nxy,Nuv,Nuv,nbands]=size(lf);

nuv=floor(Nuv/2);
nxy=floor(Nxy/2);

Nxyeven=(Nxy/2==nxy);
Nuveven=(Nuv/2==nuv);

Nuvex=Nuv+Nuveven;

FS=zeros([Nxy,Nxy,nbands,Nuvex],'single');


addfilesdown=ceil(nuv/2);
Nxyex=Nxy+2*addfilesdown+Nxyeven;
nxyex=nxy+addfilesdown;



weights_pass1=zeros([Nxyex,Nuvex,Nuvex]);
weights_pass2=zeros([Nxyex,Nuvex,Nuvex]);
fftshiftedzarr_pass1=zeros([Nxyex,Nuvex,2*Nuvex-1]);
fftshiftedzarr_pass2=zeros([Nxyex,2*Nuvex-1,Nuvex]);
shifteda=ifftshift(-nxyex:nxyex);

for a=0:Nxyex-1
    aa=shifteda(a+1);
    prez=exp(pi*i*[1:2*nuv].^2*(-aa/(nuv*Nxyex)));
    conjprez=conj(prez(1:nuv));
    shiftedzarr=[1 prez fliplr(prez)];
    fftshiftedzarr=fft(shiftedzarr,[],2);  
    weights_pass1(a+1,:,:)=repmat([fliplr(conjprez) 1 conjprez],[Nuvex,1]);
    weights_pass2(a+1,:,:)=reshape(weights_pass1(a+1,:,:),[Nuvex,Nuvex]).';
    fftshiftedzarr_pass1(a+1,:,:)=repmat(fftshiftedzarr,[Nuvex,1]);
    fftshiftedzarr_pass2(a+1,:,:)=reshape(fftshiftedzarr_pass1(a+1,:,:),[Nuvex,2*Nuvex-1]).';
end


zpimdata=zeros([Nxyex,Nxyex]);
beginzpim=addfilesdown+1+Nxyeven;
endzpim=Nxyex-addfilesdown;
zplensdata=zeros([Nuvex,Nuvex]);


for k=1:nbands
    fprintf('Generating band %d\n',k);

    S1=zeros([Nxyex,Nxyex,Nuv,Nuv]);

    fprintf('Generating S1... ');

    for u=-nuv:nuv-Nuveven
        for v=-nuv:nuv-Nuveven
            zpimdata(beginzpim:endzpim,beginzpim:endzpim)=reshape(lf(:,:,u+nuv+1,v+nuv+1,k),[Nxy,Nxy]);
            S1(:,:,u+nuv+1,v+nuv+1)=fft2(ifftshift(zpimdata));
        end
    end 
    
    fprintf('Done\n');
    S2=zeros(Nxyex,Nxyex,Nuvex,Nuvex);
    
    
    fprintf('Generating S2... ');
    for a=0:Nxyex-1
        for b=0:Nxyex-1
         
   
            shiftedyarr=zeros([Nuvex,2*Nuvex-1]);
            zplensdata(1:Nuvex-Nuveven,1:Nuvex-Nuveven)=reshape(S1(a+1,b+1,:,:),[Nuvex-Nuveven,Nuvex-Nuveven]);
            cc=zplensdata.*reshape(weights_pass1(b+1,:,:),[Nuvex,Nuvex]);
            shiftedyarr(:,1:nuv+1)=cc(:,nuv+1:Nuvex);
            shiftedyarr(:,3*nuv+2:2*Nuvex-1)=cc(:,1:nuv);    
 
            tmp0=ifft(fft(shiftedyarr,[],2).*reshape(fftshiftedzarr_pass1(b+1,:,:),[Nuvex,2*Nuvex-1]),[],2); 
            tmp1=[tmp0(:,3*nuv+2:2*Nuvex-1) tmp0(:,1:1+nuv)];
            pass1=tmp1.*reshape(weights_pass1(b+1,:,:),[Nuvex,Nuvex]);

            
            shiftedyarr=zeros([2*Nuvex-1,Nuvex]);
            cc=pass1.*reshape(weights_pass2(a+1,:,:),[Nuvex,Nuvex]);
            shiftedyarr(1:nuv+1,:)=cc(nuv+1:Nuvex,:);
            shiftedyarr(3*nuv+2:2*Nuvex-1,:)=cc(1:nuv,:);    
 
            tmp0=ifft(fft(shiftedyarr,[],1).*reshape(fftshiftedzarr_pass2(a+1,:,:),[2*Nuvex-1,Nuvex]),[],1); 
            tmp1=[tmp0(3*nuv+2:2*Nuvex-1,:); tmp0(1:1+nuv,:)];
            S2(a+1,b+1,:,:)=tmp1.*reshape(weights_pass2(a+1,:,:),[Nuvex,Nuvex]);
            
        end
    end
    
    clear S1
       
    fprintf('Done\n');
    

    S3=zeros(Nxyex,Nxyex,Nuvex);
    
    fprintf('Generating S3... ');
    for p=-nuv:nuv
        S3(:,:,p+nuv+1)=fftshift(ifft2(reshape(S2(:,:,p+nuv+1,p+nuv+1),[Nxyex,Nxyex])));
    end
    fprintf('Done\n');
    
    clear S2

    FS(:,:,k,:)=single(real(S3(beginzpim:endzpim,beginzpim:endzpim,:))/((Nuvex-Nuveven)*(Nuvex-Nuveven)));
    
    clear S3
    fprintf('band %d Done\n',k);
end    


return;